@@ -1,9 +1,23 @@
+# ********************************************************************
+#
+#  $Id: helloworld.py 32630 2018-10-10 14:11:07Z seb $
+#
+#  An example that show how to use a  Yocto-GPS
+#
+#  You can find more information on our web site:
+#   Yocto-GPS documentation:
+#      https://www.yoctopuce.com/EN/products/yocto-gps/doc.html
+#   Python API Reference:
+#      https://www.yoctopuce.com/EN/doc/reference/yoctolib-python-EN.html
+#
+# *********************************************************************
+
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 import os, sys
 # add ../../Sources to the PYTHONPATH
-sys.path.append(os.path.join("..","..","Sources"))
 sys.path.append(os.path.join("..", "..", "Sources"))
+
 from yoctopuce.yocto_api import *
 from yoctopuce.yocto_gps import *
 
@@ -38,11 +52,13 @@ if target == 'any':
 else:
     gps = YGps.FindGps(target + '.gps')
 
-while True:
-    if not (gps.isOnline()):
-        die('device not connected')
-    if (gps.get_isFixed() != YGps.ISFIXED_TRUE):
-        print("Fixed")
+if not (gps.isOnline()):
+    die('device not connected')
+
+while gps.isOnline():
+    if gps.get_isFixed() != YGps.ISFIXED_TRUE:
+        print("Fixing...")
     else:
         print(gps.get_latitude() + "  " + gps.get_longitude())
     YAPI.Sleep(1000)
+YAPI.FreeAPI()
