@@ -1,9 +1,23 @@
+# ********************************************************************
+#
+#  $Id: helloworld.py 32628 2018-10-10 13:37:59Z seb $
+#
+#  An example that show how to use a  Yocto-RS232
+#
+#  You can find more information on our web site:
+#   Yocto-RS232 documentation:
+#      https://www.yoctopuce.com/EN/products/yocto-rs232/doc.html
+#   Python API Reference:
+#      https://www.yoctopuce.com/EN/doc/reference/yoctolib-python-EN.html
+#
+# *********************************************************************
+
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 import os, sys
 # add ../../Sources to the PYTHONPATH
-sys.path.append(os.path.join("..","..","Sources"))
 sys.path.append(os.path.join("..", "..", "Sources"))
+
 from yoctopuce.yocto_api import *
 from yoctopuce.yocto_serialport import *
 
@@ -36,6 +50,6 @@ while True:
     serialPort.writeLine(line)
     YAPI.Sleep(500)
     line = serialPort.readLine()
-    if (line != ""):
+    if line != "":
         print("Received: " + line)
 YAPI.FreeAPI()
