@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # ********************************************************************
 #
-#  $Id: yocto_colorledcluster.py 38899 2019-12-20 17:21:03Z mvuilleu $
+#  $Id: yocto_colorledcluster.py 44921 2021-05-06 08:03:05Z mvuilleu $
 #
 #  Implements yFindColorLedCluster(), the high-level API for ColorLedCluster functions
 #
@@ -72,6 +72,7 @@ class YColorLedCluster(YFunction):
     COMMAND_INVALID = YAPI.INVALID_STRING
     LEDTYPE_RGB = 0
     LEDTYPE_RGBW = 1
+    LEDTYPE_WS2811 = 2
     LEDTYPE_INVALID = -1
     #--- (end of YColorLedCluster definitions)
 
@@ -138,8 +139,8 @@ class YColorLedCluster(YFunction):
         """
         Returns the RGB LED type currently handled by the device.
 
-        @return either YColorLedCluster.LEDTYPE_RGB or YColorLedCluster.LEDTYPE_RGBW, according to the RGB
-        LED type currently handled by the device
+        @return a value among YColorLedCluster.LEDTYPE_RGB, YColorLedCluster.LEDTYPE_RGBW and
+        YColorLedCluster.LEDTYPE_WS2811 corresponding to the RGB LED type currently handled by the device
 
         On failure, throws an exception or returns YColorLedCluster.LEDTYPE_INVALID.
         """
@@ -156,8 +157,8 @@ class YColorLedCluster(YFunction):
         Remember to call the matching module
         saveToFlash() method to save the setting permanently.
 
-        @param newval : either YColorLedCluster.LEDTYPE_RGB or YColorLedCluster.LEDTYPE_RGBW, according to
-        the RGB LED type currently handled by the device
+        @param newval : a value among YColorLedCluster.LEDTYPE_RGB, YColorLedCluster.LEDTYPE_RGBW and
+        YColorLedCluster.LEDTYPE_WS2811 corresponding to the RGB LED type currently handled by the device
 
         @return YAPI.SUCCESS if the call succeeds.
 
