@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #*********************************************************************
 #*
-#* $Id: yocto_display.py 38899 2019-12-20 17:21:03Z mvuilleu $
+#* $Id: yocto_display.py 43619 2021-01-29 09:14:45Z mvuilleu $
 #*
 #* Implements yFindDisplay(), the high-level API for Display functions
 #*
@@ -282,13 +282,13 @@ class YDisplayLayer(object):
         @param x : the distance from left of layer to the text anchor point, in pixels
         @param y : the distance from top of layer to the text anchor point, in pixels
         @param anchor : the text anchor point, chosen among the YDisplayLayer.ALIGN enumeration:
-                YDisplayLayer.ALIGN.TOP_LEFT,    YDisplayLayer.ALIGN.CENTER_LEFT,   
+                YDisplayLayer.ALIGN.TOP_LEFT,         YDisplayLayer.ALIGN.CENTER_LEFT,
                 YDisplayLayer.ALIGN.BASELINE_LEFT,    YDisplayLayer.ALIGN.BOTTOM_LEFT,
-                YDisplayLayer.ALIGN.TOP_CENTER,  YDisplayLayer.ALIGN.CENTER,        
+                YDisplayLayer.ALIGN.TOP_CENTER,       YDisplayLayer.ALIGN.CENTER,
                 YDisplayLayer.ALIGN.BASELINE_CENTER,  YDisplayLayer.ALIGN.BOTTOM_CENTER,
-                YDisplayLayer.ALIGN.TOP_DECIMAL, YDisplayLayer.ALIGN.CENTER_DECIMAL,
+                YDisplayLayer.ALIGN.TOP_DECIMAL,      YDisplayLayer.ALIGN.CENTER_DECIMAL,
                 YDisplayLayer.ALIGN.BASELINE_DECIMAL, YDisplayLayer.ALIGN.BOTTOM_DECIMAL,
-                YDisplayLayer.ALIGN.TOP_RIGHT,   YDisplayLayer.ALIGN.CENTER_RIGHT,  
+                YDisplayLayer.ALIGN.TOP_RIGHT,        YDisplayLayer.ALIGN.CENTER_RIGHT,
                 YDisplayLayer.ALIGN.BASELINE_RIGHT,   YDisplayLayer.ALIGN.BOTTOM_RIGHT.
         @param text : the text string to draw
 
@@ -595,9 +595,9 @@ class YDisplay(YFunction):
         self._layerHeight = YDisplay.LAYERHEIGHT_INVALID
         self._layerCount = YDisplay.LAYERCOUNT_INVALID
         self._command = YDisplay.COMMAND_INVALID
+        self._allDisplayLayers = []
         #--- (end of generated code: YDisplay attributes)
         self._sequence = ""
-        self._allDisplayLayers = []
         self._recording = False
 
     #--- (generated code: YDisplay implementation)
@@ -1047,6 +1047,31 @@ class YDisplay(YFunction):
         self.flushLayers()
         return self.sendCommand("E" + str(int(layerIdA)) + "," + str(int(layerIdB)))
 
+    def get_displayLayer(self, layerId):
+        """
+        Returns a YDisplayLayer object that can be used to draw on the specified
+        layer. The content is displayed only when the layer is active on the
+        screen (and not masked by other overlapping layers).
+
+        @param layerId : the identifier of the layer (a number in range 0..layerCount-1)
+
+        @return an YDisplayLayer object
+
+        On failure, throws an exception or returns None.
+        """
+        # layercount
+        # idx
+        layercount = self.get_layerCount()
+        if not ((layerId >= 0) and (layerId < layercount)):
+            self._throw(YAPI.INVALID_ARGUMENT, "invalid DisplayLayer index")
+            return None
+        if len(self._allDisplayLayers) == 0:
+            idx = 0
+            while idx < layercount:
+                self._allDisplayLayers.append(YDisplayLayer(self, idx))
+                idx = idx + 1
+        return self._allDisplayLayers[layerId]
+
     def nextDisplay(self):
         """
         Continues the enumeration of displays started using yFirstDisplay().
@@ -1067,29 +1092,6 @@ class YDisplay(YFunction):
 
 #--- (end of generated code: YDisplay implementation)
 
-    def get_displayLayer(self, layerId):
-        """
-        Returns a YDisplayLayer object that can be used to draw on the specified
-        layer. The content is displayed only when the layer is active on the
-        screen (and not masked by other overlapping layers).
-
-        @param layerId : the identifier of the layer (a number in range 0..layerCount-1)
-
-        @return an YDisplayLayer object
-
-        On failure, throws an exception or returns None.
-        """
-        layercount = self.get_layerCount()
-        if (layerId < 0) or (layerId >= layercount):
-            self._throw(-1, "invalid DisplayLayer index, valid values are [0.." + str(layercount - 1) + "]")
-            return None
-
-        if len(self._allDisplayLayers) == 0:
-            for i in range(0, layercount):
-                self._allDisplayLayers.append(YDisplayLayer(self, str(i)))
-
-        return self._allDisplayLayers[layerId]
-
     def flushLayers(self):
         if self._allDisplayLayers is not None:
             for it in self._allDisplayLayers:
