@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # ********************************************************************
 #
-#  $Id: yocto_refframe.py 38899 2019-12-20 17:21:03Z mvuilleu $
+#  $Id: yocto_refframe.py 43619 2021-01-29 09:14:45Z mvuilleu $
 #
 #  Implements yFindRefFrame(), the high-level API for RefFrame functions
 #
@@ -50,8 +50,8 @@ class YRefFrame(YFunction):
     """
     The YRefFrame class is used to setup the base orientation of the Yoctopuce inertial
     sensors. Thanks to this, orientation functions relative to the earth surface plane
-    can use the proper reference frame. The class also implements a tridimensional
-    sensor calibration process, which can compensate for local variations
+    can use the proper reference frame. For some devices, the class also implements a
+    tridimensional sensor calibration process, which can compensate for local variations
     of standard gravity and improve the precision of the tilt sensors.
 
     """
@@ -79,6 +79,9 @@ class YRefFrame(YFunction):
     FUSIONMODE_M4G = 2
     FUSIONMODE_COMPASS = 3
     FUSIONMODE_IMU = 4
+    FUSIONMODE_INCLIN_90DEG_1G8 = 5
+    FUSIONMODE_INCLIN_90DEG_3G6 = 6
+    FUSIONMODE_INCLIN_10DEG = 7
     FUSIONMODE_INVALID = -1
     #--- (end of YRefFrame definitions)
 
@@ -195,11 +198,12 @@ class YRefFrame(YFunction):
 
     def get_fusionMode(self):
         """
-        Returns the BNO055 fusion mode. Note this feature is only availabe on Yocto-3D-V2.
+        Returns the sensor fusion mode. Note that available sensor fusion modes depend on the sensor type.
 
         @return a value among YRefFrame.FUSIONMODE_NDOF, YRefFrame.FUSIONMODE_NDOF_FMC_OFF,
-        YRefFrame.FUSIONMODE_M4G, YRefFrame.FUSIONMODE_COMPASS and YRefFrame.FUSIONMODE_IMU corresponding
-        to the BNO055 fusion mode
+        YRefFrame.FUSIONMODE_M4G, YRefFrame.FUSIONMODE_COMPASS, YRefFrame.FUSIONMODE_IMU,
+        YRefFrame.FUSIONMODE_INCLIN_90DEG_1G8, YRefFrame.FUSIONMODE_INCLIN_90DEG_3G6 and
+        YRefFrame.FUSIONMODE_INCLIN_10DEG corresponding to the sensor fusion mode
 
         On failure, throws an exception or returns YRefFrame.FUSIONMODE_INVALID.
         """
@@ -212,11 +216,13 @@ class YRefFrame(YFunction):
 
     def set_fusionMode(self, newval):
         """
-        Change the BNO055 fusion mode. Note: this feature is only availabe on Yocto-3D-V2.
+        Change the sensor fusion mode. Note that available sensor fusion modes depend on the sensor type.
         Remember to call the matching module saveToFlash() method to save the setting permanently.
 
         @param newval : a value among YRefFrame.FUSIONMODE_NDOF, YRefFrame.FUSIONMODE_NDOF_FMC_OFF,
-        YRefFrame.FUSIONMODE_M4G, YRefFrame.FUSIONMODE_COMPASS and YRefFrame.FUSIONMODE_IMU
+        YRefFrame.FUSIONMODE_M4G, YRefFrame.FUSIONMODE_COMPASS, YRefFrame.FUSIONMODE_IMU,
+        YRefFrame.FUSIONMODE_INCLIN_90DEG_1G8, YRefFrame.FUSIONMODE_INCLIN_90DEG_3G6 and
+        YRefFrame.FUSIONMODE_INCLIN_10DEG
 
         @return YAPI.SUCCESS if the call succeeds.
 
@@ -269,12 +275,12 @@ class YRefFrame(YFunction):
         pitch/roll tilt sensors.
 
         @return a value among the YRefFrame.MOUNTPOSITION enumeration
-                (YRefFrame.MOUNTPOSITION.BOTTOM,   YRefFrame.MOUNTPOSITION.TOP,
+                (YRefFrame.MOUNTPOSITION.BOTTOM,  YRefFrame.MOUNTPOSITION.TOP,
                 YRefFrame.MOUNTPOSITION.FRONT,    YRefFrame.MOUNTPOSITION.RIGHT,
                 YRefFrame.MOUNTPOSITION.REAR,     YRefFrame.MOUNTPOSITION.LEFT),
                 corresponding to the installation in a box, on one of the six faces.
 
-        On failure, throws an exception or returns YRefFrame.MOUNTPOSITION.INVALID.
+        On failure, throws an exception or returns YRefFrame.MOUNTPOSITION_INVALID.
         """
         # position
         position = self.get_mountPos()
@@ -296,7 +302,7 @@ class YRefFrame(YFunction):
                 On the bottom face, the 12H orientation points to the front, while
                 on the top face, the 12H orientation points to the rear.
 
-        On failure, throws an exception or returns YRefFrame.MOUNTORIENTATION.INVALID.
+        On failure, throws an exception or returns YRefFrame.MOUNTORIENTATION_INVALID.
         """
         # position
         position = self.get_mountPos()
@@ -313,7 +319,7 @@ class YRefFrame(YFunction):
         the earth surface) so that the measures are made relative to this position.
 
         @param position : a value among the YRefFrame.MOUNTPOSITION enumeration
-                (YRefFrame.MOUNTPOSITION.BOTTOM,   YRefFrame.MOUNTPOSITION.TOP,
+                (YRefFrame.MOUNTPOSITION.BOTTOM,  YRefFrame.MOUNTPOSITION.TOP,
                 YRefFrame.MOUNTPOSITION.FRONT,    YRefFrame.MOUNTPOSITION.RIGHT,
                 YRefFrame.MOUNTPOSITION.REAR,     YRefFrame.MOUNTPOSITION.LEFT),
                 corresponding to the installation in a box, on one of the six faces.
